from PySide6.QtWidgets import QPushButton, QHBoxLayout, QVBoxLayout, QWidget, QLabel, QSizePolicy
from PySide6 import QtCore
from src.data import Message
from src import sendkeys
from src.data import log


class TradeWidget(QWidget):
    """docstring for TradeWidget."""

    def __init__(self,
                 message: Message,
                 parent):
        super(TradeWidget, self).__init__(parent=parent)
        self.message = message
        self.parent = parent

        self.main_layout = QHBoxLayout()
        self.main_layout.setSpacing(0)
        # self.main_layout.setMargin(0)
        self.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.setContentsMargins(0, 0, 0, 0)
        self.setGeometry(0, 0, 0, 20)
        self.setLayout(self.main_layout)
        assert message.trade
        self.label_user = QLabel(message.user)
        self.label_item = QLabel(message.trade.item)
        self.label_price = QLabel(f'{message.trade.amount} {message.trade.currency}')

        buttons = [('inv', self.inv_callback),
                   ('trade', self.trade_callback),
                   ('ty', self.thank_callback),
                   ('wait', self.wait_callback),
                   ('X', self.delete_callback)]
        for button in buttons:
            pb = QPushButton(button[0])
            pb.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
            pb.setContentsMargins(0, 0, 0, 0)
            pb.clicked.connect(button[1])
            self.main_layout.addWidget(pb)

        self.main_layout.addWidget(self.label_item)
        self.main_layout.addWidget(self.label_price)
        self.main_layout.addWidget(self.label_user)

    def inv_callback(self):
        sendkeys.invite(message=self.message)
        sendkeys.send_to_format(type='pickup', message=self.message)

    def thank_callback(self):
        sendkeys.send_to_format(type='ty', message=self.message)
        sendkeys.kick(message=self.message)
        self.delete_callback()

    def trade_callback(self):
        sendkeys.trade(message=self.message)

    def wait_callback(self):
        sendkeys.send_to_format(type='wait', message=self.message)

    def delete_callback(self):
        self.deleteLater()
        self.parent.del_trade(self.message)


class TradeCollection(QWidget):
    """docstring for TradeCollection."""

    def __init__(self,
                 message: Message,
                 parent):
        super(TradeCollection, self).__init__(parent)
        self.parent = parent
        assert message.trade
        self.unique_item = message.trade.unique_item()
        self.setSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Minimum)
        self.setGeometry(0, 0, 20, 20)
        self.main_layout = QVBoxLayout()
        self.main_layout.setSpacing(0)
        self.main_layout.setContentsMargins(0, 0, 0, 0)
        self.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.main_layout)
        trade = TradeWidget(message, self)
        self.main_layout.addWidget(trade)
        self.trades = {message.unique_trade(): trade}

    def childEvent(self, event: QtCore.QChildEvent) -> None:
        if event.removed():
            log.debug('Child removed')
            self.parent.updateSizes()
        if event.added():
            log.debug('Child added')
        return super().childEvent(event)

    def resizeEvent(self, event):
        # self.resized.emit()
        log.debug('resizeEvent')
        return super(TradeCollection, self).resizeEvent(event)

    def add_trade(self, message: Message) -> None:
        assert message.trade
        if message.unique_trade() in self.trades:
            return
        trade = TradeWidget(message, self)
        self.main_layout.addWidget(trade)
        self.trades[message.unique_trade()] = trade

    def del_trade(self, message: Message):
        assert message.trade
        del self.trades[message.unique_trade()]
        if not self.trades:
            self.parent.del_collection(self.unique_item)
            self.deleteLater()
